@@ -1,8 +1,6 @@
 #
 # Copyright 2014 eNovance
 #
-# Authors: Mehdi Abaakouk <mehdi.abaakouk@enovance.com>
-#
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
@@ -19,15 +17,19 @@ import os
 import uuid
 
 from gnocchiclient import exceptions as gnocchi_exc
+from gnocchiclient import utils as gnocchi_utils
+from keystoneauth1 import exceptions as ka_exceptions
 import mock
 from oslo_config import fixture as config_fixture
 from oslo_utils import fileutils
 from oslotest import mockpatch
 import requests
 import six
+from stevedore import extension
 import testscenarios
 
 from ceilometer.dispatcher import gnocchi
+from ceilometer.publisher import utils
 from ceilometer import service as ceilometer_service
 from ceilometer.tests import base
 
@@ -49,6 +51,7 @@ class DispatcherTest(base.BaseTestCase):
         self.resource_id = str(uuid.uuid4())
         self.samples = [{
             'counter_name': 'disk.root.size',
+            'counter_unit': 'GB',
             'counter_type': 'gauge',
             'counter_volume': '2',
             'user_id': 'test_user',
@@ -61,9 +64,11 @@ class DispatcherTest(base.BaseTestCase):
                 'image_ref': 'imageref!',
                 'instance_flavor_id': 1234,
                 'display_name': 'myinstance',
-            }},
+                }
+            },
             {
                 'counter_name': 'disk.root.size',
+                'counter_unit': 'GB',
                 'counter_type': 'gauge',
                 'counter_volume': '2',
                 'user_id': 'test_user',
@@ -78,6 +83,9 @@ class DispatcherTest(base.BaseTestCase):
                     'display_name': 'myinstance',
                 }
             }]
+        for sample in self.samples:
+            sample['message_signature'] = utils.compute_signature(
+                sample, self.conf.conf.publisher.telemetry_secret)
 
         ks_client = mock.Mock(auth_token='fake_token')
         ks_client.projects.find.return_value = mock.Mock(
@@ -85,6 +93,7 @@ class DispatcherTest(base.BaseTestCase):
         self.useFixture(mockpatch.Patch(
             'ceilometer.keystone_client.get_client',
             return_value=ks_client))
+        self.ks_client = ks_client
         self.conf.conf.dispatcher_gnocchi.filter_service_activity = True
 
     def test_config_load(self):
@@ -95,6 +104,26 @@ class DispatcherTest(base.BaseTestCase):
         self.assertIn('instance', names)
         self.assertIn('volume', names)
 
+    def test_match(self):
+        resource = {
+            'metrics':
+                ['image', 'image.size', 'image.download', 'image.serve'],
+            'attributes':
+                {'container_format': 'resource_metadata.container_format',
+                 'disk_format': 'resource_metadata.disk_format',
+                 'name': 'resource_metadata.name'},
+            'event_delete': 'image.delete',
+            'event_attributes': {'id': 'resource_id'},
+            'resource_type': 'image'}
+        plugin_manager = extension.ExtensionManager(
+            namespace='ceilometer.event.trait.trait_plugin')
+        rd = gnocchi.ResourcesDefinition(
+            resource, self.conf.conf.dispatcher_gnocchi.archive_policy,
+            plugin_manager)
+        operation = rd.event_match("image.delete")
+        self.assertEqual('delete', operation)
+        self.assertEqual(True, rd.metric_match('image'))
+
     @mock.patch('ceilometer.dispatcher.gnocchi.LOG')
     def test_broken_config_load(self, mylog):
         contents = [("---\n"
@@ -128,36 +157,56 @@ class DispatcherTest(base.BaseTestCase):
             self.assertEqual(0, len(d.resources_definition))
 
     @mock.patch('ceilometer.dispatcher.gnocchi.GnocchiDispatcher'
-                '._process_resource')
-    def _do_test_activity_filter(self, expected_samples,
-                                 fake_process_resource):
-
-        def assert_samples(resource_id, metric_grouped_samples):
-            samples = []
-            for metric_name, s in metric_grouped_samples:
-                samples.extend(list(s))
-            self.assertEqual(expected_samples, samples)
-
-        fake_process_resource.side_effect = assert_samples
+                '._if_not_cached')
+    @mock.patch('ceilometer.dispatcher.gnocchi.GnocchiDispatcher'
+                '.batch_measures')
+    def _do_test_activity_filter(self, expected_measures, fake_batch, __):
 
         d = gnocchi.GnocchiDispatcher(self.conf.conf)
         d.record_metering_data(self.samples)
-
-        fake_process_resource.assert_called_with(self.resource_id,
-                                                 mock.ANY)
+        fake_batch.assert_called_with(
+            mock.ANY, mock.ANY,
+            {'metrics': 1, 'resources': 1, 'measures': expected_measures})
 
     def test_activity_filter_match_project_id(self):
         self.samples[0]['project_id'] = (
             'a2d42c23-d518-46b6-96ab-3fba2e146859')
-        self._do_test_activity_filter([self.samples[1]])
+        self._do_test_activity_filter(1)
+
+    @mock.patch('ceilometer.dispatcher.gnocchi.LOG')
+    def test_activity_gnocchi_project_not_found(self, logger):
+        self.ks_client.projects.find.side_effect = ka_exceptions.NotFound
+        self._do_test_activity_filter(2)
+        logger.warning.assert_called_with('gnocchi project not found in '
+                                          'keystone, ignoring the '
+                                          'filter_service_activity option')
 
     def test_activity_filter_match_swift_event(self):
         self.samples[0]['counter_name'] = 'storage.api.request'
         self.samples[0]['resource_id'] = 'a2d42c23-d518-46b6-96ab-3fba2e146859'
-        self._do_test_activity_filter([self.samples[1]])
+        self._do_test_activity_filter(1)
 
     def test_activity_filter_nomatch(self):
-        self._do_test_activity_filter(self.samples)
+        self._do_test_activity_filter(2)
+
+    @mock.patch('ceilometer.dispatcher.gnocchi.GnocchiDispatcher'
+                '.batch_measures')
+    def test_unhandled_meter(self, fake_batch):
+        samples = [{
+            'counter_name': 'unknown.meter',
+            'counter_unit': 'GB',
+            'counter_type': 'gauge',
+            'counter_volume': '2',
+            'user_id': 'test_user',
+            'project_id': 'test_project',
+            'source': 'openstack',
+            'timestamp': '2014-05-08 20:23:48.028195',
+            'resource_id': 'randomid',
+            'resource_metadata': {}
+        }]
+        d = gnocchi.GnocchiDispatcher(self.conf.conf)
+        d.record_metering_data(samples)
+        self.assertEqual(0, len(fake_batch.call_args[0][1]))
 
 
 class MockResponse(mock.NonCallableMock):
@@ -179,6 +228,7 @@ class DispatcherWorkflowTest(base.BaseTestCase,
         ('disk.root.size', dict(
             sample={
                 'counter_name': 'disk.root.size',
+                'counter_unit': 'GB',
                 'counter_type': 'gauge',
                 'counter_volume': '2',
                 'user_id': 'test_user',
@@ -219,6 +269,7 @@ class DispatcherWorkflowTest(base.BaseTestCase,
         ('hardware.ipmi.node.power', dict(
             sample={
                 'counter_name': 'hardware.ipmi.node.power',
+                'counter_unit': 'W',
                 'counter_type': 'gauge',
                 'counter_volume': '2',
                 'user_id': 'test_user',
@@ -251,30 +302,36 @@ class DispatcherWorkflowTest(base.BaseTestCase,
             resource_type='ipmi')),
     ]
 
-    worflow_scenarios = [
-        ('normal_workflow', dict(measure=204, post_resource=None, metric=None,
-                                 measure_retry=None, patch_resource=204)),
-        ('new_resource', dict(measure=404, post_resource=204, metric=None,
-                              measure_retry=204, patch_resource=204)),
-        ('new_resource_fail', dict(measure=404, post_resource=500, metric=None,
-                                   measure_retry=None, patch_resource=None)),
-        ('resource_update_fail', dict(measure=204, post_resource=None,
-                                      metric=None, measure_retry=None,
-                                      patch_resource=500)),
-        ('new_metric', dict(measure=404, post_resource=None, metric=204,
-                            measure_retry=204, patch_resource=204)),
-        ('new_metric_fail', dict(measure=404, post_resource=None, metric=500,
-                                 measure_retry=None, patch_resource=None)),
-        ('retry_fail', dict(measure=404, post_resource=409, metric=None,
-                            measure_retry=500, patch_resource=None)),
-        ('measure_fail', dict(measure=500, post_resource=None, metric=None,
-                              measure_retry=None, patch_resource=None)),
+    default_workflow = dict(resource_exists=True,
+                            metric_exists=True,
+                            post_measure_fail=False,
+                            create_resource_fail=False,
+                            create_metric_fail=False,
+                            update_resource_fail=False,
+                            retry_post_measures_fail=False)
+    workflow_scenarios = [
+        ('normal_workflow', {}),
+        ('new_resource', dict(resource_exists=False)),
+        ('new_resource_fail', dict(resource_exists=False,
+                                   create_resource_fail=True)),
+        ('resource_update_fail', dict(update_resource_fail=True)),
+        ('new_metric', dict(metric_exists=False)),
+        ('new_metric_fail', dict(metric_exists=False,
+                                 create_metric_fail=True)),
+        ('retry_fail', dict(resource_exists=False,
+                            retry_post_measures_fail=True)),
+        ('measure_fail', dict(post_measure_fail=True)),
     ]
 
     @classmethod
     def generate_scenarios(cls):
+        workflow_scenarios = []
+        for name, wf_change in cls.workflow_scenarios:
+            wf = cls.default_workflow.copy()
+            wf.update(wf_change)
+            workflow_scenarios.append((name, wf))
         cls.scenarios = testscenarios.multiply_scenarios(cls.sample_scenarios,
-                                                         cls.worflow_scenarios)
+                                                         workflow_scenarios)
 
     def setUp(self):
         super(DispatcherWorkflowTest, self).setUp()
@@ -299,6 +356,8 @@ class DispatcherWorkflowTest(base.BaseTestCase,
         )
 
         self.sample['resource_id'] = str(uuid.uuid4()) + "/foobar"
+        self.sample['message_signature'] = utils.compute_signature(
+            self.sample, self.conf.conf.publisher.telemetry_secret)
 
     @mock.patch('ceilometer.dispatcher.gnocchi.LOG')
     @mock.patch('gnocchiclient.v1.client.Client')
@@ -314,89 +373,111 @@ class DispatcherWorkflowTest(base.BaseTestCase,
         # encode the resource_id
         resource_id = self.sample['resource_id']  # .replace("/", "%2F"),
         metric_name = self.sample['counter_name']
+        gnocchi_id = gnocchi_utils.encode_resource_id(resource_id)
 
         expected_calls = [
             mock.call.capabilities.list(),
-            mock.call.metric.add_measures(metric_name,
-                                          self.measures_attributes,
-                                          resource_id)]
+            mock.call.metric.batch_resources_metrics_measures(
+                {gnocchi_id: {metric_name: self.measures_attributes}})
+        ]
+        expected_debug = [
+            mock.call('gnocchi project found: %s',
+                      'a2d42c23-d518-46b6-96ab-3fba2e146859'),
+        ]
 
-        add_measures_side_effect = []
-
-        if self.measure == 404 and self.post_resource:
-            add_measures_side_effect += [
-                gnocchi_exc.ResourceNotFound(404)]
-        elif self.measure == 404 and self.metric:
-            add_measures_side_effect += [
-                gnocchi_exc.MetricNotFound(404)]
-        elif self.measure == 500:
-            add_measures_side_effect += [Exception('boom!')]
-
-        if self.post_resource:
+        measures_posted = False
+        batch_side_effect = []
+        if self.post_measure_fail:
+            batch_side_effect += [Exception('boom!')]
+        elif not self.resource_exists or not self.metric_exists:
+            batch_side_effect += [
+                gnocchi_exc.BadRequest(
+                    400, "Unknown metrics: %s/%s" % (gnocchi_id,
+                                                     metric_name))]
             attributes = self.postable_attributes.copy()
             attributes.update(self.patchable_attributes)
             attributes['id'] = self.sample['resource_id']
             attributes['metrics'] = dict((metric_name, {})
                                          for metric_name in self.metric_names)
+            for k, v in six.iteritems(attributes['metrics']):
+                if k == 'disk.root.size':
+                    v['unit'] = 'GB'
+                    continue
+                if k == 'hardware.ipmi.node.power':
+                    v['unit'] = 'W'
+                    continue
             expected_calls.append(mock.call.resource.create(
                 self.resource_type, attributes))
-            if self.post_resource == 409:
+
+            if self.create_resource_fail:
+                fakeclient.resource.create.side_effect = [Exception('boom!')]
+            elif self.resource_exists:
                 fakeclient.resource.create.side_effect = [
                     gnocchi_exc.ResourceAlreadyExists(409)]
-            elif self.post_resource == 500:
-                fakeclient.resource.create.side_effect = [Exception('boom!')]
 
-        if self.metric:
-            expected_calls.append(mock.call.metric.create({
-                'name': self.sample['counter_name'],
-                'resource_id': resource_id}))
-            if self.metric == 409:
-                fakeclient.metric.create.side_effect = [
-                    gnocchi_exc.NamedMetricAreadyExists(409)]
-            elif self.metric == 500:
-                fakeclient.metric.create.side_effect = [Exception('boom!')]
+                expected_calls.append(mock.call.metric.create({
+                    'name': self.sample['counter_name'],
+                    'unit': self.sample['counter_unit'],
+                    'resource_id': resource_id}))
+                if self.create_metric_fail:
+                    fakeclient.metric.create.side_effect = [Exception('boom!')]
+                elif self.metric_exists:
+                    fakeclient.metric.create.side_effect = [
+                        gnocchi_exc.NamedMetricAlreadyExists(409)]
+                else:
+                    fakeclient.metric.create.side_effect = [None]
+
+            else:  # not resource_exists
+                expected_debug.append(mock.call(
+                    'Resource %s created', self.sample['resource_id']))
+
+            if not self.create_resource_fail and not self.create_metric_fail:
+                expected_calls.append(
+                    mock.call.metric.batch_resources_metrics_measures(
+                        {gnocchi_id: {metric_name: self.measures_attributes}})
+                )
+
+                if self.retry_post_measures_fail:
+                    batch_side_effect += [Exception('boom!')]
+                else:
+                    measures_posted = True
 
-        if self.measure_retry:
-            expected_calls.append(mock.call.metric.add_measures(
-                metric_name,
-                self.measures_attributes,
-                resource_id))
-            if self.measure_retry == 204:
-                add_measures_side_effect += [None]
-            elif self.measure_retry == 500:
-                add_measures_side_effect += [
-                    Exception('boom!')]
         else:
-            add_measures_side_effect += [None]
+            measures_posted = True
 
-        if self.patch_resource and self.patchable_attributes:
+        if measures_posted:
+            batch_side_effect += [None]
+            expected_debug.append(
+                mock.call("%(measures)d measures posted against %(metrics)d "
+                          "metrics through %(resources)d resources", dict(
+                              measures=len(self.measures_attributes),
+                              metrics=1, resources=1))
+            )
+
+        if self.patchable_attributes:
             expected_calls.append(mock.call.resource.update(
                 self.resource_type, resource_id,
                 self.patchable_attributes))
-            if self.patch_resource == 500:
+            if self.update_resource_fail:
                 fakeclient.resource.update.side_effect = [Exception('boom!')]
+            else:
+                expected_debug.append(mock.call(
+                    'Resource %s updated', self.sample['resource_id']))
 
-        fakeclient.metric.add_measures.side_effect = add_measures_side_effect
+        batch = fakeclient.metric.batch_resources_metrics_measures
+        batch.side_effect = batch_side_effect
 
         self.dispatcher.record_metering_data([self.sample])
 
         # Check that the last log message is the expected one
-        if (self.measure == 500 or self.measure_retry == 500 or
-                self.metric == 500 or self.post_resource == 500 or
-                (self.patch_resource == 500 and self.patchable_attributes)):
+        if (self.post_measure_fail or self.create_metric_fail
+                or self.create_resource_fail
+                or self.retry_post_measures_fail
+                or (self.update_resource_fail and self.patchable_attributes)):
             logger.error.assert_called_with('boom!', exc_info=True)
-        elif self.patch_resource == 204 and self.patchable_attributes:
-            logger.debug.assert_called_with(
-                'Resource %s updated', self.sample['resource_id'])
+        else:
             self.assertEqual(0, logger.error.call_count)
-        elif self.measure == 200:
-            logger.debug.assert_called_with(
-                "Measure posted on metric %s of resource %s",
-                self.sample['counter_name'],
-                self.sample['resource_id'])
-            self.assertEqual(0, logger.error.call_count)
-
         self.assertEqual(expected_calls, fakeclient.mock_calls)
-
+        self.assertEqual(expected_debug, logger.debug.mock_calls)
 
 DispatcherWorkflowTest.generate_scenarios()
